use std::collections::BTreeMap;

use anyhow::Error;
use cozo::DbInstance;

pub fn add_schema(db: &DbInstance) -> anyhow::Result<()> {
    let params = BTreeMap::new();
    match db.run_script(
        "
    {:create apps {
        id: String,
        =>
        last_access: Int,
        name: String,
        description: String,
    }}
    {:create peers {
        id: String,
        =>
        name: String,
    }}
    {:create elements {
        id: String,
        =>
        content: Json,
        latest_message: String?,
        local_changes: Bool,
    }}
    {:create tags {
        tag: String,
        element: String,
    }}
    ",
        params,
        cozo::ScriptMutability::Mutable,
    ) {
        Ok(_) => Ok(()),
        Err(e) => Err(Error::msg(format!("Failed to set up schema: {}", e))),
    }
}
