use serde::{Deserialize, Serialize};

use super::{ElementContent, ElementId, MessageId};

#[derive(Serialize, Deserialize, Clone, Debug)]
pub struct Element {
    // Uuid identifying the element itself
    id: ElementId,
    content: ElementContent,
    latest_message: Option<MessageId>,
    local_changes: bool,
}

impl From<(ElementId, ElementContent, Option<MessageId>, bool)> for Element {
    fn from(value: (ElementId, ElementContent, Option<MessageId>, bool)) -> Self {
        Element {
            id: value.0,
            content: value.1,
            latest_message: value.2,
            local_changes: value.3,
        }
    }
}

impl Element {
    pub fn new(id: ElementId, content: ElementContent) -> Self {
        // A new element with no latest message must have local changes
        Element {
            id: id,
            content: content,
            latest_message: None,
            local_changes: true,
        }
    }

    pub fn id(&self) -> &ElementId {
        &self.id
    }
    pub fn content(&self) -> &ElementContent {
        &self.content
    }
}
